package xic.analysis;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import xic.Statistics;
import xic.analysis.AvailableCopies.CPPair;
import xic.analysis.AvailableCopies.CPValue;
import xic.analysis.DataFlowAnalysis.FlowInfo;
import xic.ir.IRExpr;
import xic.ir.IRStmt;
import xic.ir.IRTemp;
import xic.ir.StructHashSet.StructWrapper;
import xic.visitors.RewritingVisitor;


// Implementation of Copy Propagation
public class CopyPropagation {
	public static void transform(Statistics stats, int pass, Map<String, CFGNode> cfgs) {
		Map<StructWrapper, IRExpr> replaceMap = new HashMap<StructWrapper, IRExpr>();
		RewritingVisitor visitor = new RewritingVisitor(replaceMap);

		for (String funcName : cfgs.keySet()) {
			CFGNode headNode = cfgs.get(funcName);
			Set<CFGNode> nodeSet = headNode.flatten();
			AvailableCopies ac = new AvailableCopies(headNode.flatten());
			ac.solve();
			int numReplacements = 0;

			// replace a temp x in IRStmt with y if (x,y) is in the pair list
			for (CFGNode node : nodeSet) {
				if (node.value() instanceof IRStmt) {
					IRStmt stmt = (IRStmt) node.value();
					FlowInfo info = ac.getFlowInfo(node);

					replaceMap.clear();
					for (CPPair pair : ((CPValue) info.entry).pairs())
						replaceMap.put(new StructWrapper(new IRTemp(pair.v1)), 
								new IRTemp(pair.v2));
					visitor.setKnownExpressions(replaceMap);

					node.setValue(visitor.visit(stmt));
					numReplacements += visitor.getNumRewrites();
				}
			}
			if (numReplacements > 0)
				stats.log("copy", "Pass " + pass + ": '" + funcName
						+ "': propagated " + numReplacements + " copies");
		}
	}
}
