package xic.ast;

import java.util.Arrays;

import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.xi.Position;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Describes assignment statements. The lvalue is of type
 * <code>Expr</code> to support assignments to array entries.
 */
public class Assignment extends Stmt {
	private Expr lvalue;
	private Expr rvalue;

	public Assignment(Position pos, Expr lvalue, Expr rvalue) {
		super(pos);
		this.lvalue = lvalue;
		this.rvalue = rvalue;
	}

	/**
	 * Copy constructor, makes a shallow copy while
	 * providing new position information.
	 */
	public Assignment(Position pos, Assignment assignment) {
		super(pos);
		lvalue = assignment.lvalue();
		rvalue = assignment.rvalue();
	}

	public Expr lvalue() {
		return lvalue;
	}
	
	public Expr rvalue() {
		return rvalue;
	}

	public Iterable<VisualizableTreeNode> children() {
		return Arrays.asList(new VisualizableTreeNode[] { lvalue, rvalue });
	}

    public ASTNode visitChildren(NodeVisitor v) {
    	Expr lvalue = (Expr) v.visit(this, this.lvalue);
		Expr rvalue = (Expr) v.visit(this, this.rvalue);

		if (lvalue != this.lvalue || rvalue != this.rvalue)
			return new Assignment(position, lvalue, rvalue);

		return this;
	}
    
    public Copyable copy() {
		return new Assignment(position, lvalue, rvalue);
    }
	
	public String label() {
		return "assign";
	}
}
