package xic.ast;

import java.util.Arrays;

import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.xi.Position;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Represents either a function call or an array indexing
 * operation -- the two will later be distinguished when
 * additional semantic information is available. To support
 * indexing into multidimensional arrays, the <code>target</tt>
 * attribute is of type <code>Expr</code> (as opposed to
 * limiting it to an identifier).
 */
public class Call extends Expr {
	public enum CallType {
		UNSPECIFIED,
		FUNCTION_CALL,
		INDEXING_OPERATION
	};

	private Expr target;
	private TupleExpr param;
	private CallType callType;

	public Call(Position pos, Expr target, Expr param, CallType callType) {
		super(pos);
		this.target = target;
		if (param instanceof TupleExpr)
			this.param = (TupleExpr) param;
		else
			this.param = new TupleExpr(param);
		this.callType = callType;
	}
	
	public Call(Position pos, Expr target, Expr param) {
		this(pos, target, param, CallType.UNSPECIFIED);
	}

	public Expr name() {
		return target;
	}
	
	public Expr target() {
		return target;
	}

	public TupleExpr param() {
		return param;
	}
	
	public CallType callType() {
		return callType;
	}

	public Iterable<VisualizableTreeNode> children() {
		return Arrays.asList(new VisualizableTreeNode[] {
			target, param
		});
	}
	
    public ASTNode visitChildren(NodeVisitor v) {
		Expr target = (Expr) v.visit(this, this.target);
		TupleExpr param = (TupleExpr) v.visit(this, this.param);

		if (target != this.target || param != this.param)
			return new Call(position, target, param, callType); 

		return this;
	}
    
    public Copyable copy() {
    	Expr expr = new Call(position, target, param, callType);
    	expr.type = type;
    	expr.dataSource = dataSource;
    	return expr;
    }

	public String label() {
		return "call";
	}
}
