package xic.ast;

import java.util.Arrays;

import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.xi.Position;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Expression used to represent the length keyword
 */
public class DotExpr extends Expr {
	private Expr target;
	private String member;

	public DotExpr(Position pos, Expr target, String member) {
		super(pos);
		this.target = target;
		this.member = member;
	}

	public Expr target() {
		return target;
	}

	public String member() {
		return member;
	}

	public Iterable<VisualizableTreeNode> children() {
		return Arrays.asList(new VisualizableTreeNode[] {
				target
		});
	}

    public ASTNode visitChildren(NodeVisitor v) {
    	Expr target = (Expr) v.visit(this, this.target);

    	if(target != this.target)
			return new DotExpr(position, target, member);

		return this;
	}
    
    public Copyable copy() {
		Expr expr = new DotExpr(position, this.target, this.member);
    	expr.type = type;
    	expr.dataSource = dataSource;
    	return expr;
    }

	public String label() {
		return "dotExpr " + member;
	}
}
