package xic.ast;

import xic.ErrorSink;
import xic.visitors.NodeVisitor;
import edu.cornell.cs.cs4120.xi.CompilationException;
/**
 * Special 'invalid' type used to propagage errors while type
 * checking -- this effectively allows to continue checking without
 * producing reduntant errors due to incomplete information.
 */
public class InvalidType extends TypeNode {
	public static final InvalidType INVALID_TYPE = new InvalidType();

	public InvalidType() {
	}
	
	public Copyable copy() {
		return new InvalidType();
	}
	
	public ASTNode visitChildren(NodeVisitor v) {
		return this;
	}

	public String label() {
		return "invalid";
	}
	
	public String toString() {
		return "invalid";
	}
	
	public boolean isIdenticalType(TypeNode node) {
		throw new UnsupportedOperationException("InvalidType.isIdenticalType()");
	}

	public boolean isSubtypeOf(TypeNode node) {
		throw new UnsupportedOperationException("InvalidType.isSubtypeOf()");
	}

	public int size() {
		throw new UnsupportedOperationException("InvalidType.size()");
	}
	
	public void match (TypeNode node, ErrorSink sink) {
		sink.addError(new CompilationException("Unmatched type, excepted: "
				+this.label()+", got: "+node.label(), node.position()));
	}
}
