package xic.ast;

import xic.ErrorSink;
import xic.visitors.NodeVisitor;
import edu.cornell.cs.cs4120.xi.CompilationException;
import edu.cornell.cs.cs4120.xi.Position;

/**
 * TypeNode corresponding to a primitive type
 * such as integer or boolean.
 */
public class PrimitiveType extends TypeNode {
	/* Static types without position information */
	public static final PrimitiveType INTEGER_TYPE = 
		new PrimitiveType(Type.INTEGER);
	public static final PrimitiveType BOOLEAN_TYPE = 
		new PrimitiveType(Type.BOOLEAN);

	public enum Type {
		INTEGER,
		BOOLEAN
	};

	private Type type = null;
	
	public PrimitiveType(Position pos, Type type) {
		super(pos);
		this.type = type;
	}

	public PrimitiveType(Type type) {
		this.type = type;
	}

	public Type type() {
		return type;
	}
	
	public boolean isSubtypeOf(TypeNode node) {
		if (node instanceof PrimitiveType) {
			PrimitiveType ptnode = (PrimitiveType) node;
			return ptnode.type() == this.type;
		} else if (node instanceof TupleType) {
			TupleType tnode = (TupleType) node;
			if (tnode.elementTypes().size() == 1)
				return isSubtypeOf(tnode.elementTypes().get(0));
			else
				return false;
		} else if (node instanceof ArrayType) {
			ArrayType tnode = (ArrayType) node;
			return isSubtypeOf(tnode.elementType());
		} else if (node instanceof UnitType) {
			return true;
		} else {
			return false;
		}
	}

	public boolean isIdenticalType(TypeNode node) {
		if (!(node instanceof PrimitiveType))
			return false;
		
		PrimitiveType other = (PrimitiveType) node;
		return other.type == type;
	}

	public String label() {
		switch (type) {
			case INTEGER: return "int";
			case BOOLEAN: return "bool";
			default: return "invalid";
		}
	}
	
	public String toString() {
		return label();
	}

	public ASTNode visitChildren(NodeVisitor v) {
		return this;
	}

	public Copyable copy() {
		return new PrimitiveType (position, this.type);
	}

	public int size() {
		return 1;
	}
	
	public void match (TypeNode node, ErrorSink sink) {
		if (node instanceof PrimitiveType) {
			PrimitiveType pt = (PrimitiveType) node;
			if (this.type!=pt.type)
				sink.addError(new CompilationException("Unmatched primitive type, excepted: "
						+(this.type==Type.INTEGER?"Int":"Bool")+", got: "+
						(pt.type==Type.INTEGER?"Int":"Bool"), node.position()));
		}
		else
			sink.addError(new CompilationException("Unmatched type, excepted: "
					+this.label()+", got: "+node.label(), node.position()));
	}
}
