package xic.ast;

import java.util.Arrays;

import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.xi.Position;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Represents a while statement in the source code.
 * A loop condition expression and a body statement 
 * are both mandatory.
 */
public class While extends Stmt {
	private Expr cond;
	private Stmt body;

	public While(Position pos, Expr cond, Stmt body) {
		super(pos);
		this.cond = cond;
		this.body = body;
	}

	public Expr cond() {
		return cond;
	}
	
	public Stmt body() {
		return body;
	}

	public Iterable<VisualizableTreeNode> children() {
		return Arrays.asList(new VisualizableTreeNode[] {
			cond, body
		});
	}

    public ASTNode visitChildren(NodeVisitor v) {
		Expr cond = (Expr) v.visit(this, this.cond);
		Stmt body = (Stmt) v.visit(this, this.body);

		if (cond != this.cond || body != this.body)
			return new While(position, cond, body); 

		return this;
	}
    
    public Copyable copy() {
		return new While(position, cond, body); 
    }

	public String label() {
		return "while";
	}
}