package xic.codegen.matchers;

import java.util.ArrayList;
import java.util.List;

import xic.Configuration;
import xic.InternalCompilerException;
import xic.Configuration.Target;
import xic.codegen.AbstractTile;
import xic.codegen.AsmDirective;
import xic.codegen.InstrList;
import xic.codegen.LabelInstr;
import xic.codegen.Matcher;
import xic.codegen.Tile;
import xic.codegen.Tiler;
import xic.ir.IRConst;
import xic.ir.IRDataSeq;
import xic.ir.IRExpr;
import xic.ir.IRName;
import xic.ir.IRNode;
import xic.ir.NameGenerator;

public class DataSeqMatcher implements Matcher {
	protected static class DataSeqTile extends AbstractTile {
		private List<Object> data;
		private String name;
		private boolean readOnly;

		public DataSeqTile(String name, boolean readOnly, List<Object> data) {
			this.name = name;
			this.data = data;
			this.readOnly = readOnly;
		}

		public int localCost() {
			return 0;
		}

		public String translate(InstrList il, NameGenerator ng) {
			boolean osx = Configuration.TARGET == Target.DARWIN;

			if (readOnly)
				il.add(new AsmDirective(osx ? "\t.const_data" : "\t.section .rodata"));
			else
				il.add(new AsmDirective(osx ? "\t.data" : "\t.section .data"));

			il.add(new AsmDirective("\t.align "+ (osx ? Configuration.LOG_WORD_SIZE : Configuration.WORD_SIZE)));

			if (name.startsWith("_"))
				il.add(new AsmDirective(".globl " + name));
			il.add(new LabelInstr(name));
			for (Object o : data) {
				if (Configuration.WORD_SIZE == 4) {
					il.add(new AsmDirective("\t.long " + o));
				} else {
					il.add(new AsmDirective("\t.quad " + o));
				}
			}
			il.add(new AsmDirective("\t.text"));
			il.add(new AsmDirective(""));

			return null;
		}
	};

	public Tile match(IRNode node, Tiler tiler) {
		if (node instanceof IRDataSeq) {
			IRDataSeq dSeq = (IRDataSeq) node;
			List<Object> values = new ArrayList<Object>();

			for (IRExpr c : dSeq.values()) {
				if (c instanceof IRConst)
					values.add(((IRConst) c).value());
				else if (c instanceof IRName)
					values.add(((IRName) c).name());
				else
					throw new InternalCompilerException("Unknown data type!");
			}
			
			return new DataSeqTile(dSeq.name(), dSeq.isReadOnly(), values);
		}
		
		return null;
	}
}
