package xic.codegen.matchers;

import java.util.List;

import xic.codegen.AbstractTile;
import xic.codegen.Instr;
import xic.codegen.InstrList;
import xic.codegen.Matcher;
import xic.codegen.Tile;
import xic.codegen.Tiler;
import xic.codegen.Instr.Op;
import xic.codegen.matchers.AddrModeHelper.AddrMode;
import xic.ir.IRMem;
import xic.ir.IRNode;
import xic.ir.NameGenerator;


public class MemMatcher implements Matcher {
	protected static class MemTile extends AbstractTile {
		private AddrMode addrMode;

		public MemTile(AddrMode addrMode) {
			this.addrMode = addrMode;
		}

		public int localCost() {
			return 1 + addrMode.estimateCost();
		}

		public String translate(InstrList il, NameGenerator ng) {
			String newTemp = ng.nextTemp();

			il.add(new Instr("mov",
					new Instr.Op(Instr.Op.Type.REG, newTemp),
					addrMode.toOp(il, ng)
				));
			return newTemp;
		}

		public Op getOp(InstrList il, NameGenerator ng) {
			return addrMode.toOp(il, ng);
		}
	};
	
	
	public Tile match(IRNode node, Tiler tiler) {
		if (!(node instanceof IRMem))
			return null;

		IRMem mem = (IRMem) node;
		Tile bestTile = null;
		int bestCost = Integer.MAX_VALUE;

		List<AddrMode> modes = AddrModeHelper.findMemAddressingModes(tiler, mem);

		for (AddrMode addrMode : modes) {
			Tile tile = new MemTile(addrMode);
			int cost = tile.estimateCost();
			if (cost < bestCost) {
				bestTile = tile;
				bestCost = cost;
			}
		}

		return bestTile;
	}
}
