package xic.ir;

import java.util.Arrays;
import java.util.Set;

import xic.ast.ASTNode;
import xic.ast.Copyable;
import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Evaluate an expression and discard its result
 */
public class IRExp extends IRStmt {
	private IRExpr expr;
	
	public IRExp(IRExpr expr) {
		this.expr = expr;
	}

	public IRExpr expr() {
		return expr;
	}

	public Iterable<VisualizableTreeNode> children() {
		return Arrays.asList(new VisualizableTreeNode[] {
			expr
		});
	}

    public ASTNode visitChildren(NodeVisitor v) {
    	IRExpr expr = (IRExpr) v.visit(this, this.expr);

		if (expr != this.expr)
			return new IRExp(expr);

		return this;
	}

	public String label() {
		return "EXP";
	}
	
	public boolean containsCalls() {
		return expr.containsCalls();
	}
	
	public boolean containsMemReads() {
		return expr.containsMemReads();
	}

	public boolean containsMemWrites() {
		return expr.containsMemWrites();
	}

	public void findReadTemps(Set<String> tempNames) {
		expr.findReadTemps(tempNames);
	}

	public void findWrittenTemps(Set<String> tempNames) {
		expr.findWrittenTemps(tempNames);
	}

	public int computeMaximumCallArguments() {
		return expr.computeMaximumCallArguments();
	}

	public int computeMaximumCallResults() {
		return expr.computeMaximumCallResults();
	}

	public int nodeCount() {
		return expr.nodeCount() + 1;
	}

	public Copyable copy() {
		return new IRExp(expr);
	}
	
	public Copyable deepCopy() {
		return new IRExp((IRExpr) expr.deepCopy());
	}
	
	public boolean equalsTree(Object object) {
		if (!(object instanceof IRExp))
			return false;
		return ((IRExp) object).expr.equalsTree(expr);
	}
	
	public int treeHashCode() {
		return expr.treeHashCode();
	}
}