package xic.ir;

import java.util.Arrays;
import java.util.Set;

import xic.ast.ASTNode;
import xic.ast.Copyable;
import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

public class IRFuncDecl extends IRNode {
	private String name;
	private IRStmt body;
	private boolean isProcedure;

	public IRFuncDecl(String name, IRStmt stmt, boolean isProcedure) {
		this.name = name;
		this.body = stmt;
		this.isProcedure = isProcedure;
	}

	public Copyable copy() {
		return new IRFuncDecl(name, body, isProcedure);
	}
	
    public Copyable deepCopy() {
    	return new IRFuncDecl(name, (IRStmt) body.deepCopy(), isProcedure);
    }

	public String name() {
		return name;
	}
	
	public IRStmt body() {
		return body;
	}

	public String label() {
		return "FUNC " + name; 
	}

	public Iterable<VisualizableTreeNode> children() {
		return Arrays.asList(new VisualizableTreeNode[] {
			body
		});
	}

	public boolean containsCalls() {
		return body.containsCalls();
	}

	public boolean containsMemReads() {
		return body.containsMemReads();
	}

	public boolean containsMemWrites() {
		return body.containsMemWrites();
	}

	public void findReadTemps(Set<String> tempNames) {
		body.findReadTemps(tempNames);
	}

	public void findWrittenTemps(Set<String> tempNames) {
		body.findWrittenTemps(tempNames);
	}
	
	public int nodeCount() {
		return 1 + body.nodeCount();
	}

    public ASTNode visitChildren(NodeVisitor v) {
    	IRStmt stmt = (IRStmt) v.visit(this, this.body);

		if (stmt != this.body)
			return new IRFuncDecl(name, stmt, isProcedure);

		return this;
	}
    
	public boolean equalsTree(Object object) {
		if (!(object instanceof IRFuncDecl))
			return false;
		IRFuncDecl other = (IRFuncDecl) object;
		return name.equals(other.name) && body.equals(other.body);
	}

	public int treeHashCode() {
		return 17 + name.hashCode() + body.treeHashCode() * 37;
	}

	public boolean isProcedure() {
		return isProcedure;
	}

	public int computeMaximumCallArguments() {
		return body.computeMaximumCallArguments();
	}

	public int computeMaximumCallResults() {
		return body.computeMaximumCallResults();
	}
}
