package xic.parser;

import java.io.Reader;

import xic.ErrorSink;
import xic.PositionImpl;
import xic.ast.CompUnit;
import xic.lexer.XiLexer;
import edu.cornell.cs.cs4120.xi.AbstractSyntaxNode;
import edu.cornell.cs.cs4120.xi.CompilationException;
import edu.cornell.cs.cs4120.xi.parser.Parser;

public class ParserImpl implements Parser {
	/// Supported languages
	public enum Language {
		// Iota/09
		IOTA_09,
		/// Xi (2011)
		XI
	};

	private Reader reader;
	private Language language;
	private String source;

	public ParserImpl(Language language, Reader reader, String source) {
		this.reader = reader;
		this.language = language;
		this.source = source;
	}

	public AbstractSyntaxNode parse() throws CompilationException {
		try {
			ErrorSink errorSink = new ErrorSink();
			CompUnit compUnit = null;
			XiLexer lexer = new XiLexer(source, TokenType.SOURCE_IDENTIFIER, reader);

			if (language == Language.IOTA_09) {
				compUnit = new IotaParser(lexer).doParse(errorSink);
			} else if (language == Language.XI) {
				compUnit = new XiParser(lexer).doParse(errorSink);
			} else {
				throw new CompilationException("Unknown language!",
					new PositionImpl());
			}
				
			if (errorSink.hasErrors()) {
				/* This interface only supports returning the first error */
				throw errorSink.getErrors().get(0);
			}
			return compUnit;
		} catch (CompilationException e) {
			throw e;
		} catch (Exception e) {
			/* An unexpected exception was caught and no associated position record
			   is available -> wrap into an CompilationException with a
			   position record clearly stating that it is invalid */
			throw new CompilationException(e.getMessage(), new PositionImpl());
		}
	}
	
	public AbstractSyntaxNode parse(ErrorSink errorSink) throws CompilationException {
		CompUnit compUnit = null;
		XiLexer lexer = new XiLexer(source, TokenType.SOURCE_IDENTIFIER, reader);

		if (language == Language.IOTA_09) {
			compUnit = new IotaParser(lexer).doParse(errorSink);
		} else if (language == Language.XI) {
			compUnit = new XiParser(lexer).doParse(errorSink);
		} else {
			throw new CompilationException("Unknown language!",
				new PositionImpl());
		}
			
		return compUnit;
	}
}
