package xic.visitors;

import java.util.List;

import xic.ErrorSink;
import xic.ast.ASTNode;

import edu.cornell.cs.cs4120.xi.CompilationException;

/**
 * Visitor with error handling support - collects any 
 * CompilationExceptions in a special error sink and proceeds
 * with the traversal.
 */
public abstract class ErrorHandlingVisitor extends SelectiveVisitor {
	protected ErrorSink errorSink;
	
	public ErrorHandlingVisitor(ErrorSink sink) {
		this.errorSink = sink;
	}
	
	protected ErrorHandlingVisitor(ErrorSink sink, List<Class<? extends ASTNode>> ignoredTypes) {
		super(ignoredTypes);
		this.errorSink = sink;
	}


	public ErrorSink getErrorSink() {
		return errorSink;
	}

	/* To be implemented by a subclasses */
	protected abstract NodeVisitor enterNode(ASTNode parent, ASTNode n) 
		throws CompilationException;

	/* To be implemented by subclasses */
	protected abstract ASTNode leaveNode(ASTNode parent, ASTNode n, 
				ASTNode nNew, NodeVisitor vNew)
		throws CompilationException;

	/**
	 * Call the implementation leaveNode() method and possibly
	 * collect errors
	 */
	protected ASTNode leave(ASTNode parent, ASTNode n, ASTNode nNew,
			NodeVisitor vNew) {
		try {
			return leaveNode(parent, n, nNew, vNew);
		} catch (CompilationException e) {
			errorSink.addError(e);
			return nNew;
		}
	}

	protected NodeVisitor enter(ASTNode parent, ASTNode n) {
		try {
			return enterNode(parent, n);
		} catch (CompilationException e) {
			errorSink.addError(e);
			return this;
		}
	}
}
