package xic;

import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.junit.Test;

import xic.ast.ASTNode;
import xic.ast.FuncDecl;
import xic.ir.NameMangler;
import xic.parser.IotaParser;

import edu.cornell.cs.cs4120.xi.CompilationException;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Test cases for encoding function declarations as specified in
 * Iota9 High Level ABI Specification
 */
public class NameManglerTest {
	List<FuncDecl> decls = new ArrayList<FuncDecl>();
	
	private void findFuncDecls ( VisualizableTreeNode node) {
		Iterable<VisualizableTreeNode> childrens = node.children();
		Iterator<VisualizableTreeNode> iterator = childrens.iterator();
		while (iterator.hasNext()) {
			VisualizableTreeNode child = iterator.next();
			if (child instanceof FuncDecl)
				decls.add( (FuncDecl) child);
			else
				findFuncDecls(child);
				
		}
	}
	
	// return the encoded name for function declarations
	private String encodingCheck(String input) throws CompilationException {
		String result="";
		ASTNode parsedAST = IotaParser.getAST(input);
		findFuncDecls (parsedAST);
		
		if (decls.size() == 0)
			System.out.println( "Empty");
		for (int i=0; i<decls.size(); i++) {
			FuncDecl decl = decls.get(i);
			result += NameMangler.getEncoding(decl, null);
		}
		return result;
	}

	@Test
	public void test1() {
		decls.clear();
		assertEquals ( encodingCheck("main(args: int[][]) = {\n" + "}"),
				"_Imain_paai");
	}
	
	@Test
	public void test2() {
		decls.clear();
		assertEquals ( encodingCheck("unparseInt(n:int):int[] = {}"),
				"_IunparseInt_aii");
	}
	
	@Test
	public void test3() {
		decls.clear();
		assertEquals ( encodingCheck("parseInt(str:int[]):(int,bool) = {}"),
				"_IparseInt_t2ibai");
	}
	
	@Test
	public void test4() {
		decls.clear();
		assertEquals ( encodingCheck("eof():bool = {}"),
				"_Ieof_b");
	}
	
	@Test
	public void test5() {
		decls.clear();
		assertEquals ( encodingCheck("gcd(a:int,b:int):int = {}"),
				"_Igcd_iii");
	}
	
	@Test
	public void test6() {
		decls.clear();
		assertEquals ( encodingCheck("an_example(a:((int,int),(bool,bool,bool))):int[] = {}"),
				"_Ian__example_ait2t2iit3bbb");
	}
	
	@Test
	public void test7() {
		decls.clear();
		assertEquals ( encodingCheck("multiple__underScores() = {}"),
				"_Imultiple____underScores_p");
	}
}